//
//  Marble.swift
//  FaceTilt
//
//  Created by Sally McNichols on 10/26/14.
//  Copyright (c) 2014 CMU iOS Development StuCo. All rights reserved.
//

import UIKit

struct MarbleAcceleration {
    var xAccel: Double
    var yAccel: Double
}

class Marble: NSObject {
    // constants to adjust acceleration (we scale it down quite a bit)
    let kMarbleAccelerationConstant: Double = 1.0 / 500.0
    let kMarbleBouncingConstant: Double = 0.65 // should be >= 0 (and hopefully < 1).
    let kMarbleRotationConstant: Double = 1.0 / 100.0
    
    var size: CGSize
    var origin: CGPoint
    var xVelocity: Double
    var yVelocity: Double
    var zRotation: Double // represents an angle in radians
    
    init(size: CGSize) {
        self.size = size
        self.origin = CGPointZero
        self.xVelocity = 0.0
        self.yVelocity = 0.0
        self.zRotation = 0.0
        super.init()
    }
    
    func center() -> CGPoint {
        return CGPointMake(self.origin.x + self.size.width / 2.0, self.origin.y + self.size.height / 2.0)
    }
    
    func setCenter(center: CGPoint) -> Void {
        self.origin = CGPointMake(center.x - self.size.width / 2.0, center.y - self.size.height / 2.0)
    }
   
    func accelerateInX(xAccel: Double, yAccel: Double) -> Void {
        self.xVelocity += xAccel
        self.yVelocity -= yAccel
        self.checkWalls()
        self.updatePosition()
    }
    
    // check to see if the marble has hit the sides of the phone screen
    func checkWalls() -> Void {
        let screenBounds: CGSize = UIScreen.mainScreen().bounds.size
        if((self.xVelocity > 0 && self.origin.x + self.size.width >= screenBounds.width) ||
            (self.xVelocity < 0 && self.origin.x <= 0)) {
            self.xVelocity *= -self.kMarbleBouncingConstant
        }
        
        if ((self.yVelocity > 0 && self.origin.y + self.size.height >= screenBounds.height) ||
            (self.yVelocity < 0 && self.origin.y <= 0)) {
                self.yVelocity *= -self.kMarbleBouncingConstant
        }
    }
    
    func updatePosition() -> Void {
        let currCenter: CGPoint = center()
        let newCenter: CGPoint = CGPointMake(currCenter.x + CGFloat(self.xVelocity * kMarbleAccelerationConstant), currCenter.y + CGFloat(self.yVelocity * kMarbleAccelerationConstant))
        
        setCenter(newCenter)
    }
    
    func updateMarbleAcceleration(accel: MarbleAcceleration) -> Void {
        accelerateInX(accel.xAccel, yAccel: accel.yAccel)
    }
    
    func updateMarbleRotation(rotation: Double) -> Void {
        self.zRotation += rotation * kMarbleRotationConstant
    }
    
    func rotation() -> CGAffineTransform {
        return CGAffineTransformMakeRotation(CGFloat(self.zRotation))
    }
}
